#coding=utf-8
from .Amazon import MWS,Amazon_Orders,Amazon_Feeds,Amazon_Products,AuthToken,Amazon_Finance
from ..webStore_interface import BaseInterface,get_element_by_tag,parseString


class Interface(BaseInterface):
    
    def __init__(self,account):
        self._mws=MWS(account['access_key'],account['secret_key'],account['account_id'],account['mkplace_id'],account['MWSAuthToken'])
        self._account = account
    
    def _get_feed_submission_list_status(self,FeedSubmissionId):
        '''
        获取上传数据的进行状态
        @param FeedSubmitssionId: 上传数据的唯一标示
        @param amazon: 网店信息
        '''
        sub_list = Amazon_Feeds(**self._account).get_feed_submission_list(feedids=FeedSubmissionId)
        sub_list_str = parseString(sub_list)
        FeedProcessingStatus = get_element_by_tag(sub_list_str,'FeedProcessingStatus')
        return FeedProcessingStatus

    def _get_feed_submission_result_status(self,FeedSubmissionId):
        '''
        获取上传数据的返回状态
        @param FeedSubmitssionId: 上传数据的标示
        @param bulk_list: 上传的列表
        @param amazon: 网店
        '''
        sub_result_list = Amazon_Feeds(**self._account).get_feed_submission_result(feedid = FeedSubmissionId)
        sub_result_list_str = parseString(sub_result_list)
        MessagesProcessed = get_element_by_tag(sub_result_list_str,"MessagesProcessed")
        MessagesSuccessful = get_element_by_tag(sub_result_list_str,"MessagesSuccessful")
        MessagesWithError = get_element_by_tag(sub_result_list_str,"MessagesWithError")
        return_list = {}
        
        if MessagesProcessed > MessagesSuccessful or float(MessagesWithError or 0) >0:
            return_list['status'] = False
            for each in sub_result_list_str.getElementsByTagName("Result"):
                MessageID = get_element_by_tag(each,"MessageID")
                return_list[MessageID]  = {}
                return_list[MessageID]['status'] = False
                return_list[MessageID]['ResultDescription'] = get_element_by_tag(each,"ResultDescription")
                return_list[MessageID]['ResultMessageCode'] = get_element_by_tag(each,"ResultMessageCode")
        else:
            return_list['status'] = True
        return return_list
   
    def getToken(self):
        return AuthToken(**self._account).getAuthToken()
   
    def getImgUrl(self,**args):
        return Amazon_Products(**self._account).get_matching_product_for_id(self._account['mkplace_id'],**args)
     
    def getOrder(self,**args):
        return Amazon_Orders(**self._account).get_order(**args)
        
    def listOrders(self,**args):
        return Amazon_Orders(**self._account).list_orders([self._account['mkplace_id']],**args)
        
    def listOrdersMore(self,**args):
        return Amazon_Orders(**self._account).list_orders_by_next_token(**args)
        
    def listOrderItems(self,**args):
        return Amazon_Orders(**self._account).list_order_items(**args)
        
    def listOrderItemsMore(self,**args):
        return Amazon_Orders(**self._account).list_order_items_by_next_token(**args)

    def listFinaceEventGroup(self,**args):
        print args
        return Amazon_Finance(**self._account).list_finace_event_group(**args)

    def listFinaceEvent(self,**args):
        return Amazon_Finance(**self._account).list_finace_event(**args)

    def uploadTrack(self,**args):
        '''
        批量上传订单的跟踪单号
        @param amazon: Amazon实例
        @param submit_list: key-value，key为amazon calss，
                            value 为一个map，其中 'IdOrder' 我们数据库的标识  ,'OrderID' 订单号 ,'Supplier' 订单的物流方式 , 'TrackingNumber' 订单的跟踪单号
        '''
        submit_list = args['submit_list']
        from datetime import datetime
        import time
        import pytz
        
        date_time = datetime.now(pytz.UTC).strftime('%FT%T')
        my_feed_start = '''<?xml version="1.0" encoding="utf-8"?>
                                <AmazonEnvelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="amzn-envelope.xsd">
                                <Header>
                                <DocumentVersion>1.01</DocumentVersion>
                                <MerchantIdentifier>%(Account_Id)s</MerchantIdentifier>
                                </Header>
                                <MessageType>OrderFulfillment</MessageType>''' % {'Account_Id': self._account['account_id']}
                                
        my_feed_end = "\n</AmazonEnvelope>"
        
        bulk_list = {}
        bulk_list['status'] = False
        for row in submit_list:
            bulk_list[row['id']]  = {'status':False}
                
                
        my_feed_request = ''

        for row in submit_list:
            
            if bulk_list[row['id']]['status'] == True:
                continue
            print 'OrderID',row['OrderID']
            bulk_list[row['id']]['status'] = True
            bulk_list[row['id']]['id'] = row['id']
            bulk_list[row['id']]['IdOrder'] = row['IdOrder']
            bulk_list[row['id']]['OrderID'] = row['OrderID']
            bulk_list[row['id']]['Supplier'] = row['Supplier']
            bulk_list[row['id']]['TrackingNumber'] = row['TrackingNumber']
            
            
            try:
                my_feed_request = my_feed_request + '''\n<Message>
                    <MessageID>%(num)s</MessageID>
                    <OperationType>Update</OperationType>
                    <OrderFulfillment>
                    <AmazonOrderID>%(orderID)s</AmazonOrderID>
                    <FulfillmentDate>%(date_time)s</FulfillmentDate>
                    <FulfillmentData>
                    <CarrierName>%(supplier)s</CarrierName>
                    <ShipperTrackingNumber>%(track_num)s</ShipperTrackingNumber>
                    </FulfillmentData>        
                    </OrderFulfillment>
                    </Message>''' % {
                                     'num': row['id'],
                                      'orderID' : row['OrderID'],
                                      'date_time' : (date_time),
                                      'supplier' : row['Supplier'], 
                                      'track_num': row['TrackingNumber']                                         
                                                }
                
            except Exception, e:
                print 'set feed request Error:',str(e)
                return "submit wrong"
           
        my_feed = my_feed_start + my_feed_request + my_feed_end
        
        orders = Amazon_Feeds(**self._account).submit_feed(my_feed, '_POST_ORDER_FULFILLMENT_DATA_',[self._account['mkplace_id']])
        
        next_dom = parseString(orders)
        FeedSubmissionId =get_element_by_tag(next_dom,"FeedSubmissionId")
        
        print 'FeedSubmissionId:',FeedSubmissionId
        
        deal_times_temp = 0
        while deal_times_temp < 15:
            deal_times_temp = deal_times_temp +1
            feedProcessingStatus = self._get_feed_submission_list_status(FeedSubmissionId)
            if feedProcessingStatus == '_DONE_':
                break
            time.sleep(60)
        return_list = self._get_feed_submission_result_status(FeedSubmissionId)
        
        print 'return_list:',return_list
        
        if not return_list['status']:
            for each in return_list:
                if each == 'status':
                    continue
                bulk_list[each]['status'] = return_list[each]['status']
                bulk_list[each]['ResultDescription'] = return_list[each]['ResultDescription']
                bulk_list[each]['ResultMessageCode'] = return_list[each]['ResultMessageCode']
                    
        return bulk_list


    pass